/*
 * corn64.c: function definition for the corn library
 *
 * Copyright 2013 by Jerry Nettrouer II
 *
 * This file is part of the corn package.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose and without fee is hereby granted, provided
 * that the above copyright notice and this permission notice appear
 * in all copies, and that the name of Jerry Nettrouer II not be used in
 * advertising or publicity pertaining to distribution of the document
 * or software without specific, written prior permission.
 *
 * Jerry Nettrouer II makes no representations about the suitability of the
 * software described herein for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * The author may be contacted via http://www.inpito.org/
 * Jerry Nettrouer <j2[at]inpito.org>
 *
 * I am not entirely sure about these limits, but they seem to be working for now.
 * 2,305,843,009,213,693,952 is the largest safe number on a 64-bit
 * 536,870,912 is the largest safe number on a 32-bit
 * size needs set to 5 on 32-bit, but can be 9 on 64-bit
 * large_number needs set to 9 on 32-bit, but can be 18 on 64-bit
 */

#include <stdio.h>
#include <stdlib.h>

// Function protoptype for finding a modulo
long int modulo( long int larger,  long int smaller );

// Defining the corn function using big numbers
long int corn( long int max  )
{
	// Initailize the number stuff
	int whole, number, loop, out = 0, size = 5;
	long int output, random_number, integer;
	double round;

	// Add 1 to max to set our modulo output to max
	max = labs( max + 1 );

	// Initailize the read in character string
	char line[size];

	// Initailize the arrays to hold the ascii number representations
	char count_number[2] = "";
	char ascii_number[3] = "";
	char large_number[9] = "";

	// Open the /dev/urandom to read in a few characters
	FILE * fp;
	fp = fopen("/dev/urandom", "r");
	fread( line, 1, size, fp );
	fclose(fp);

	// Loop through the charaters to build a number string
	for ( loop = 0; out != 1; loop++ )
	{
		// Convert the characters to an ascii number representation
		sprintf( ascii_number, "%03d", abs ( line[loop] ) );

		// Convert the ascii 0-255 number to a 0-99 digit
		number = labs( atol ( ascii_number ) );
		round = number * 0.388235294118;
		whole = (int)(round+0.5);
		sprintf( count_number, "%02d", abs ( whole ) );

		// Append the number representations together
		sprintf( large_number, "%s%s", large_number, count_number );

		// Turn the string into a real random number
		random_number = labs( atol( large_number ) );

		// Test the number for out of range size
		if ( random_number > 536870912 )
		{ 
			printf( "Corn Went Out Of Range Processing %ld\n", random_number );
			out = 1;
		}

		/* To get a good probability range that is from 0 to max,
		   I needed a number range that is at least twice the size of max. */

		// Test the number for twice the size of the max
		if ( random_number > max * 2 )
		{	out = 1;	}

	}

	// Delimit the number to within max number range
	output = modulo( random_number,  max );

	// output the result !
	return output;
}

// I had to write this function for ARM32 and ARM64 because apparently they can NOT
// use a modulus operator or do division worth a crap
long int modulo( long int larger,  long int smaller )
{
	// Initial integers
	long int loop, count = 0;

	// Assign the value of large to the loop.
	loop = larger;

	// While the loop is greater than or equal to the smaller value
	while ( loop >= smaller )
	{
		// The loop is equal to the loop minus the value of smaller
		loop = loop - smaller;

		// Count up from lower to find our value to multiple by
		count = count + 1;
	}

	// Return the remainer of the larger number minus the smaller times the lowest
	return labs ( larger - ( smaller * count ) );
}
